/**************************************************************************//**
 * @file     startup_ARMCM23.c
 * @brief    CMSIS Core Device Startup File for
 *           ARMCM23 Device
 * @version  V1.1.0
 * @date     23. January 2019
 ******************************************************************************/
/*
 * Copyright (c) 2009-2019 Arm Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
*/

#include "gd32e23x.h"


/*----------------------------------------------------------------------------
  Linker generated Symbols
 *----------------------------------------------------------------------------*/
extern uint32_t __etext;
extern uint32_t __data_start__;
extern uint32_t __data_end__;
extern uint32_t __copy_table_start__;
extern uint32_t __copy_table_end__;
extern uint32_t __zero_table_start__;
extern uint32_t __zero_table_end__;
extern uint32_t __bss_start__;
extern uint32_t __bss_end__;
extern uint32_t __StackTop;
extern uint32_t __StackLimit;

/*----------------------------------------------------------------------------
  Exception / Interrupt Handler Function Prototype
 *----------------------------------------------------------------------------*/
typedef void( *pFunc )( void );


/*----------------------------------------------------------------------------
  External References
 *----------------------------------------------------------------------------*/
extern void _start     (void) __attribute__((noreturn)); /* PreeMain (C library entry point) */


/*----------------------------------------------------------------------------
  Internal References
 *----------------------------------------------------------------------------*/
void Default_Handler(void) __attribute__ ((noreturn));
void Reset_Handler  (void) __attribute__ ((noreturn));


/*----------------------------------------------------------------------------
  User Initial Stack & Heap
 *----------------------------------------------------------------------------*/
//<h> Stack Configuration
//  <o> Stack Size (in Bytes) <0x0-0xFFFFFFFF:8>
//</h>
#define  __STACK_SIZE  0x00000400
static uint8_t stack[__STACK_SIZE] __attribute__ ((aligned(8), used, section(".stack")));

//<h> Heap Configuration
//  <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF:8>
//</h>
#define  __HEAP_SIZE   0x00000400
#if __HEAP_SIZE > 0
static uint8_t heap[__HEAP_SIZE]   __attribute__ ((aligned(8), used, section(".heap")));
#endif


/*----------------------------------------------------------------------------
  Exception / Interrupt Handler
 *----------------------------------------------------------------------------*/
/* Exceptions */
void NMI_Handler                        (void) __attribute__ ((weak, alias("Default_Handler")));
void HardFault_Handler                  (void) __attribute__ ((weak, alias("Default_Handler")));
void SVC_Handler                        (void) __attribute__ ((weak, alias("Default_Handler")));
void PendSV_Handler                     (void) __attribute__ ((weak, alias("Default_Handler")));
void SysTick_Handler                    (void) __attribute__ ((weak, alias("Default_Handler")));

void WWDGT_IRQHandler                   (void) __attribute__ ((weak, alias("Default_Handler")));
void LVD_IRQHandler                     (void) __attribute__ ((weak, alias("Default_Handler")));
void RTC_IRQHandler                     (void) __attribute__ ((weak, alias("Default_Handler")));
void FMC_IRQHandler                     (void) __attribute__ ((weak, alias("Default_Handler")));
void RCU_IRQHandler                     (void) __attribute__ ((weak, alias("Default_Handler")));
void EXTI0_1_IRQHandler                 (void) __attribute__ ((weak, alias("Default_Handler")));
void EXTI2_3_IRQHandler                 (void) __attribute__ ((weak, alias("Default_Handler")));
void EXTI4_15_IRQHandler                (void) __attribute__ ((weak, alias("Default_Handler")));
void DMA_Channel0_IRQHandler            (void) __attribute__ ((weak, alias("Default_Handler")));
void DMA_Channel1_2_IRQHandler          (void) __attribute__ ((weak, alias("Default_Handler")));
void DMA_Channel3_4_IRQHandler          (void) __attribute__ ((weak, alias("Default_Handler")));
void ADC_CMP_IRQHandler                 (void) __attribute__ ((weak, alias("Default_Handler")));
void TIMER0_BRK_UP_TRG_COM_IRQHandler   (void) __attribute__ ((weak, alias("Default_Handler")));
void TIMER0_Channel_IRQHandler          (void) __attribute__ ((weak, alias("Default_Handler")));
void TIMER2_IRQHandler                  (void) __attribute__ ((weak, alias("Default_Handler")));
void TIMER5_IRQHandler                  (void) __attribute__ ((weak, alias("Default_Handler")));
void TIMER13_IRQHandler                 (void) __attribute__ ((weak, alias("Default_Handler")));
void TIMER14_IRQHandler                 (void) __attribute__ ((weak, alias("Default_Handler")));
void TIMER15_IRQHandler                 (void) __attribute__ ((weak, alias("Default_Handler")));
void TIMER16_IRQHandler                 (void) __attribute__ ((weak, alias("Default_Handler")));
void I2C0_EV_IRQHandler                 (void) __attribute__ ((weak, alias("Default_Handler")));
void I2C1_EV_IRQHandler                 (void) __attribute__ ((weak, alias("Default_Handler")));
void SPI0_IRQHandler                    (void) __attribute__ ((weak, alias("Default_Handler")));
void SPI1_IRQHandler                    (void) __attribute__ ((weak, alias("Default_Handler")));
void USART0_IRQHandler                  (void) __attribute__ ((weak, alias("Default_Handler")));
void USART1_IRQHandler                  (void) __attribute__ ((weak, alias("Default_Handler")));
void I2C0_ER_IRQHandler                 (void) __attribute__ ((weak, alias("Default_Handler")));
void I2C1_ER_IRQHandler                 (void) __attribute__ ((weak, alias("Default_Handler")));


/*----------------------------------------------------------------------------
  Exception / Interrupt Vector table
 *----------------------------------------------------------------------------*/
extern const pFunc __Vectors[240];
       const pFunc __Vectors[240] __attribute__((used, section(".vectors"))) = {
  (pFunc)(&__StackTop),                     /*     Initial Stack Pointer */
  Reset_Handler,                            /*     Reset Handler */
  NMI_Handler,                              /* -14 NMI Handler */
  HardFault_Handler,                        /* -13 Hard Fault Handler */
  0,                                        /*     Reserved */
  0,                                        /*     Reserved */
  0,                                        /*     Reserved */
  0,                                        /*     Reserved */
  0,                                        /*     Reserved */
  0,                                        /*     Reserved */
  0,                                        /*     Reserved */
  SVC_Handler,                              /*  -5 SVCall Handler */
  0,                                        /*     Reserved */
  0,                                        /*     Reserved */
  PendSV_Handler,                           /*  -2 PendSV Handler */
  SysTick_Handler,                          /*  -1 SysTick Handler */

  /* Interrupts */
  WWDGT_IRQHandler,                         /*   16 Window Watchdog Timer */
  LVD_IRQHandler,                           /*   17 LVD through EXTI Line detect */
  RTC_IRQHandler,                           /*   18 RTC through EXTI Line */
  FMC_IRQHandler,                           /*   19 FMC */
  RCU_IRQHandler,                           /*   20 RCU */
  EXTI0_1_IRQHandler,                       /*   21 EXTI Line 0 and EXTI Line 1 */
  EXTI2_3_IRQHandler,                       /*   22 EXTI Line 2 and EXTI Line 3 */
  EXTI4_15_IRQHandler,                      /*   23 EXTI Line 4 to EXTI Line 15 */
  0,                                        /*   Reserved */
  DMA_Channel0_IRQHandler,                  /*   25 DMA Channel 0 */
  DMA_Channel1_2_IRQHandler,                /*   26 DMA Channel 1 and DMA Channel 2 */
  DMA_Channel3_4_IRQHandler,                /*   27 DMA Channel 3 and DMA Channel 4 */
  ADC_CMP_IRQHandler,                       /*   28 ADC and Comparator */
  TIMER0_BRK_UP_TRG_COM_IRQHandler,         /*   29 TIMER0 Break,Update,Trigger and Commutation */
  TIMER0_Channel_IRQHandler,                /*   30 TIMER0 Channel Capture Compare */
  0,                                        /*   Reserved */
  TIMER2_IRQHandler,                        /*   32 TIMER2 */
  TIMER5_IRQHandler,                        /*   33 TIMER5 */
  0,                                        /*   Reserved */
  TIMER13_IRQHandler,                       /*   35:TIMER13 */
  TIMER14_IRQHandler,                       /*   36:TIMER14 */
  TIMER15_IRQHandler,                       /*   37:TIMER15 */
  TIMER16_IRQHandler,                       /*   38:TIMER16 */
  I2C0_EV_IRQHandler,                       /*   39:I2C0 Event */
  I2C1_EV_IRQHandler,                       /*   40:I2C1 Event */
  SPI0_IRQHandler,                          /*   41:SPI0 */
  SPI1_IRQHandler,                          /*   42:SPI1 */
  USART0_IRQHandler,                        /*   43:USART0 */
  USART1_IRQHandler,                        /*   44:USART1 */
  0,                                        /*   Reserved */
  0,                                        /*   Reserved */
  0,                                        /*   Reserved */
  I2C0_ER_IRQHandler,                       /*   48:I2C0 Error */
  0,                                        /*   Reserved */
  I2C1_ER_IRQHandler                        /*   50:I2C1 Error */
};


/*----------------------------------------------------------------------------
  Reset Handler called on controller reset
 *----------------------------------------------------------------------------*/
void Reset_Handler(void) {
  uint32_t *pSrc, *pDest;
  uint32_t *pTable __attribute__((unused));

  __set_MSPLIM((uint32_t)&__StackLimit);

  SystemInit();                             /* CMSIS System Initialization */

/* Firstly it copies data from read only memory to RAM.
 * There are two schemes to copy. One can copy more than one sections.
 * Another can copy only one section. The former scheme needs more
 * instructions and read-only data to implement than the latter.
 * Macro __STARTUP_COPY_MULTIPLE is used to choose between two schemes.
 */

#ifdef __STARTUP_COPY_MULTIPLE
/* Multiple sections scheme.
 *
 * Between symbol address __copy_table_start__ and __copy_table_end__,
 * there are array of triplets, each of which specify:
 *   offset 0: LMA of start of a section to copy from
 *   offset 4: VMA of start of a section to copy to
 *   offset 8: size of the section to copy. Must be multiply of 4
 *
 * All addresses must be aligned to 4 bytes boundary.
 */
  pTable = &__copy_table_start__;

  for (; pTable < &__copy_table_end__; pTable = pTable + 3) {
    pSrc  = (uint32_t*)*(pTable + 0);
    pDest = (uint32_t*)*(pTable + 1);
    for (; pDest < (uint32_t*)(*(pTable + 1) + *(pTable + 2)) ; ) {
      *pDest++ = *pSrc++;
    }
  }
#else
/* Single section scheme.
 *
 * The ranges of copy from/to are specified by following symbols
 *   __etext: LMA of start of the section to copy from. Usually end of text
 *   __data_start__: VMA of start of the section to copy to
 *   __data_end__: VMA of end of the section to copy to
 *
 * All addresses must be aligned to 4 bytes boundary.
 */
  pSrc  = &__etext;
  pDest = &__data_start__;

  for ( ; pDest < &__data_end__ ; ) {
    *pDest++ = *pSrc++;
  }
#endif /*__STARTUP_COPY_MULTIPLE */

/* This part of work usually is done in C library startup code.
 * Otherwise, define this macro to enable it in this startup.
 *
 * There are two schemes too.
 * One can clear multiple BSS sections. Another can only clear one section.
 * The former is more size expensive than the latter.
 *
 * Define macro __STARTUP_CLEAR_BSS_MULTIPLE to choose the former.
 * Otherwise define macro __STARTUP_CLEAR_BSS to choose the later.
 */
#ifdef __STARTUP_CLEAR_BSS_MULTIPLE
/* Multiple sections scheme.
 *
 * Between symbol address __copy_table_start__ and __copy_table_end__,
 * there are array of tuples specifying:
 *   offset 0: Start of a BSS section
 *   offset 4: Size of this BSS section. Must be multiply of 4
 */
  pTable = &__zero_table_start__;

  for (; pTable < &__zero_table_end__; pTable = pTable + 2) {
    pDest = (uint32_t*)*(pTable + 0);
    for (; pDest < (uint32_t*)(*(pTable + 0) + *(pTable + 1)) ; ) {
      *pDest++ = 0;
    }
  }
#elif defined (__STARTUP_CLEAR_BSS)
/* Single BSS section scheme.
 *
 * The BSS section is specified by following symbols
 *   __bss_start__: start of the BSS section.
 *   __bss_end__: end of the BSS section.
 *
 * Both addresses must be aligned to 4 bytes boundary.
 */
  pDest = &__bss_start__;

  for ( ; pDest < &__bss_end__ ; ) {
    *pDest++ = 0UL;
  }
#endif /* __STARTUP_CLEAR_BSS_MULTIPLE || __STARTUP_CLEAR_BSS */

  _start();                                 /* Enter PreeMain (C library entry point) */
}


/*----------------------------------------------------------------------------
  Default Handler for Exceptions / Interrupts
 *----------------------------------------------------------------------------*/
void Default_Handler(void) {

  while(1);
}
